bl_info = {
    "name": "NBA 2K16 Model Importer",
    "description": "Importing NBA2K16 3D Models (with Animation Skeleton) into Blender's viewport",
    "author": "gregkwaste",
    "version": (0, 9, 'beta'),
    "blender": (2, 71, 0),
    "location": "View3D > Scene",
    "warning": "",  # used for warning icon and text in addons panel
    "wiki_url": "",
    "tracker_url": "",
    "category": "Import-Export"}

import struct
import imp
import os
import webbrowser
import bpy
import bmesh
from io import BytesIO
from mathutils import Vector, Matrix
from shutil import copyfile
from math import radians

# print (os.path.dirname(os.path.abspath(__file__)))

execpath = os.path.dirname(bpy.app.binary_path)
json_parser = imp.load_source('json_parser', os.path.join(
    execpath, 'nba2k_tools', 'json_parser.py'))

version_text = 'v0.9 Coded by gregkwaste'


class nbapanel(bpy.types.Panel):
    bl_label = 'NBA2KXX Importer Panel'
    bl_idname = 'nba2kxx_importer_panel'
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = 'scene'

    def draw(self, context):
        scn = context.scene
        layout = self.layout

        row = layout.row(align=True)
        row.label(icon='INFO', text='Importer')
        box = layout.box()
        col = box.column()
        row = col.row()
        row.prop(scn, 'nba2kxx_skeleton_flag')
        row = col.row()
        row.alignment = 'EXPAND'
        row.prop(scn, 'nba_json_filepath')
        row = col.row()
        row.operator('mesh.nba2kxx_model_import')

        row = layout.row(align=True)
        row.label(icon='INFO', text='Exporter')
        box = layout.box()
        col = box.column()
        row = col.row()
        row.prop(scn, 'nba_export_filepath')
        row = col.row()
        row.operator("mesh.nba2kxx_model_export")

        row = layout.row()
        row.alignment = 'EXPAND'
        row.operator(
            "system.visit_url_blog", text='Visit Official Website', emboss=False)
        row.operator(
            "system.visit_url", text='Visit Official Thread', emboss=False)
        row.operator("system.visit_url_donate", text='Donate', emboss=False)
        row = layout.row()
        row.label(text=version_text)


class ModelEntry:

    def __init__(self):
        self.faces = []
        self.verts = []
        self.uvs = []
        self.cols = []

    def parse_data(self, ob):
        self.faces = []
        self.verts = []
        self.uvs = []
        self.cols = []

        uvs_0 = []
        uvs_1 = []
        uvs_2 = []
        data = ob.data
        print('Processing: ', ob.name)
        bm = bmesh.new()
        bm.from_mesh(data)
        bm.normal_update()

        # Matrices
        object_matrix_wrld = ob.matrix_world
        rot_x_mat = Matrix.Rotation(radians(0), 4, 'X')
        scale_mat = Matrix.Scale(1, 4)  # Keeping only this scale for FIFA 15

        # New way
        uvcount = len(bm.loops.layers.uv)
        colcount = len(bm.loops.layers.color)

        # Store Vertices
        for v in bm.verts:
            co = scale_mat * rot_x_mat * object_matrix_wrld * v.co
            self.verts.append((co[0], co[1], co[2]))

        for i in range(len(self.verts)):
            uvs_0.append(0)
            uvs_1.append(0)
            uvs_2.append(0)

        for f in bm.faces:
            for l in f.loops:
                vid = l.vert.index  # Get vertex index
                # uvs
                for k in range(uvcount):
                    layer = bm.loops.layers.uv[k]
                    u = l[layer].uv.x
                    v = 1. - l[layer].uv.y
                    exec('uvs_' + str(k) + '[vid]=(round(u,8),round(v,8))')
                # for k in range(colcount):
                    # layer=bm.loops.layers.color[k]
                    # r=l[layer].r*1023
                    # g=l[layer].g*1023
                    # b=l[layer].b*1023
                    # eval('col_'+str(k)+'.append((r,g,b))')

                # Define a face for each loop
            # if len(f.loops)==4:
                # indices.append((id,id+1,id+2))
                # indices.append((id+3,id,id+2))
                # id+=4
            # elif len(f.loops)==3:
                # indices.append((id,id+1,id+2))
                # id+=3
        # bm.free()

        # Gather different maps
        for j in range(uvcount):
            eval('self.uvs.append(uvs_' + str(j) + ')')
        # for j in range(colcount):
        #    eval('self.cols.append(col_' + str(j) + ')')

        for i in range(len(self.uvs)):
            temp = eval('uvs_' + str(i))
            for j in range(len(temp)):
                if not temp[j]:
                    temp[j] = (0.0, 0.0)


class Bone:

    def __init__(self, Sibling, Child, Name, boneId, Position):
        self.Sibling = Sibling
        self.Child = Child
        self.Name = Name
        self.id = boneId
        self.Position = Position


class Model2k:

    def __init__(self, f):
        self.__file__ = f
        self.type = struct.unpack('>I', f.read(4))[0]
        self.size = struct.unpack('<I', f.read(4))[0]
        self.hash = struct.unpack('>I', f.read(4))[0]
        self.num0 = struct.unpack('<H', f.read(2))[0]
        self.num1 = struct.unpack('B', f.read(1))[0]
        self.num2 = struct.unpack('B', f.read(1))[0]
        self.data = None
        print('\tSection ', hex(self.type), self.size, hex(
            self.hash), self.num0, self.num1, self.num2)

    def tell(self):
        return self.__file__.tell()

    def get_verts(self, vformat, scale):
        if vformat == "R16G16B16A16_SNORM":
            return self.read_vertices_half(scale)
        elif vformat == "R32G32B32_FLOAT":
            return self.read_vertices_float3(scale)
        else:
            print('\tNot Implemented vertex format: ', vformat)

    def get_colors(self, vformat):
        if vformat == "R8G8B8A8_UNORM":
            return self.read_colors_unorm()
        elif vformat == "R16G16B16A16_FLOAT":
            return self.read_colors_half()
        else:
            print('\tNot Implemented vertex format: ', vformat)

    def write_verts(self, vformat, scale, verts):
        if vformat == "R16G16B16A16_SNORM":
            return self.write_vertices_half(scale, verts)
        elif vformat == "R32G32B32_FLOAT":
            return self.write_vertices_float3(scale, verts)
        else:
            print('Not Implemented vertex format: ', vformat)

    def write_uvs(self, vformat, scale, offset, uvs):
        print('\tUV Format: ', vformat)
        print('\tScale: ', scale)
        print('\tOffset: ', offset)

        if vformat in ["R16G16_SNORM", "R16G16_FLOAT"]:
            return self.write_uvs_half(scale, offset, uvs)
        elif vformat in ["R32_UINT",  "R16G16_UINT"]:
            print('Skipping Unidentified Section')
            self.skip_section()
        else:
            print('Not Implemented uv format: ', vformat)

    def get_blendindices(self, vformat):
        if vformat == "R8G8B8A8_UINT":
            return self.read_blendindices_uint()
        else:
            print('Not Implemented vertex format: ', vformat)

    def get_blendweights(self, vformat):
        if vformat == "R8G8B8A8_UNORM":
            return self.read_blendweights_unorm()
        else:
            print('Not Implemented vertex format: ', vformat)

    def get_normals(self, vformat):
        if vformat == "R32G32B32_FLOAT":
            return self.read_vertices_float()
        elif vformat == "R16G16B16A16_SNORM":
            return self.read_normals_half()
        else:
            print('Wrong num2')

    def get_uvs(self, vformat, scale, offset):
        print('\tUVS format: ', vformat)
        if vformat in ["R16G16_SNORM", "R16G16_FLOAT"]:
            return self.read_uvs_half(scale, offset)
        elif vformat == "R16G16B16A16_FLOAT":
            return self.read_uvs_half(scale, offset)
        elif vformat == 'R32_UINT':
            return self.read_colors_32uint()
        else:
            print('\tNot Implemented uv format: ', vformat)

    def fill_normals(self, count):

        norms = []
        for i in range(count):
            norms.append((0, 0, 1))
        return norms

    def read_lists(self, start, target, base):
        indices = []
        count = 0
        self.__file__.seek(0x14 + 0x2 * start)
        while count < target:
            temp = struct.unpack('<3H', self.__file__.read(6))
            if not (temp[0] == temp[1] or temp[1] == temp[2] or temp[2] == temp[0]):
                temp = (temp[0] + base, temp[1] + base, temp[2] + base)
                indices.append(temp)
            count += 3
        return indices

    def read_strips(self, start, target, base):
        indices = []
        count = 0
        self.__file__.seek(0x14 + 0x2 * start)
        while count < target and (self.__file__.tell() < (self.size + 0x14)):
            temp = []
            s = struct.unpack('<H', self.__file__.read(2))[0]
            count += 1
            while s != 0xFFFF and count < target and (self.__file__.tell() < (self.size + 0x14)):
                temp.append(s)
                count += 1
                s = struct.unpack('<H', self.__file__.read(2))[0]
            if len(temp):
                indices.append(temp)

        print('Strips Read: ', count - 1)
        return indices

    def read_vertices_half(self, scale):
        v_count = (self.size - 0x10) // 8
        print('\tReading Vertices: ', v_count)
        verts = []
        if not scale:
            scale = [1., 1., 1., 1.]
        for i in range(v_count):
            v1 = struct.unpack('<h', self.__file__.read(2))[0] / 65535.0
            v2 = struct.unpack('<h', self.__file__.read(2))[0] / 65535.0
            v3 = struct.unpack('<h', self.__file__.read(2))[0] / 65535.0
            self.__file__.read(2)
            verts.append((-scale[0] * v1, scale[2] * v3, scale[1] * v2))
        return verts

    def read_normals_half(self):
        v_count = (self.size - 0x10) // 8
        verts = []
        for i in range(v_count):
            v1 = struct.unpack('<h', self.__file__.read(2))[0] / 65535.0
            v2 = struct.unpack('<h', self.__file__.read(2))[0] / 65535.0
            v3 = struct.unpack('<h', self.__file__.read(2))[0] / 65535.0
            self.__file__.read(2)
            verts.append((v1, v2, v3))
        return verts

    def read_colors_half(self):
        v_count = (self.size - 0x10) // 8
        verts = []
        for i in range(v_count):
            v1 = struct.unpack('<h', self.__file__.read(2))[0] / 65535.0
            v2 = struct.unpack('<h', self.__file__.read(2))[0] / 65535.0
            v3 = struct.unpack('<h', self.__file__.read(2))[0] / 65535.0
            self.__file__.read(2)
            verts.append((v1, v3, v2))
        return verts

    def write_vertices_half(self, scale, verts):
        # oldv_count = (self.size - 0x10) // 8
        if not scale:
            scale = [1., 1., 1.]
        v_count = len(verts)
        t = BytesIO()
        for i in range(v_count):
            v1 = struct.pack('<3h', int(-verts[i][0] * 65535.0 / scale[0]), int(
                verts[i][2] * 65535.0 / scale[2]), int(verts[i][1] * 65535.0 / scale[1]))
            t.write(v1)
            t.write(struct.pack('>H', 0xFF7F))
        t.seek(0)
        self.__file__.write(t.read())

    def read_vertices_float3(self, scale):
        v_count = (self.size - 0x10) // 12
        print('Reading Vertices: ', v_count)
        verts = []
        for i in range(v_count):
            v1 = struct.unpack('<f', self.__file__.read(4))[0]
            v2 = struct.unpack('<f', self.__file__.read(4))[0]
            v3 = struct.unpack('<f', self.__file__.read(4))[0]
            verts.append((-scale[0] * v1, scale[2] * v3, scale[1] * v2))
        return verts

    def write_vertices_float3(self, scale, verts):
        if not scale:
            scale = [1., 1., 1.]
        v_count = len(verts)
        t = BytesIO()
        for i in range(v_count):
            v1 = struct.pack('<3f', -verts[i][0] * scale[0],
                             verts[i][2] * scale[2], verts[i][1] * scale[1])
            t.write(v1)
        t.seek(0)
        self.__file__.write(t.read())

    def read_vertices_float(self, f, scale):
        v_count = (self.size - 0x10) // 12
        print('Reading Vertices: ', v_count)
        verts = []
        for i in range(v_count):
            v1 = struct.unpack('<f', f.read(4))[0]
            v2 = struct.unpack('<f', f.read(4))[0]
            v3 = struct.unpack('<f', f.read(4))[0]
            verts.append((scale[0] * v1, scale[2] * v3, scale[1] * v2))
        return verts

    def read_uvs_half(self, scale, offset):
        uv_count = (self.size - 0x10) // 4
        uvs = []
        if not scale:
            scale = [1., 1., 1., 1.]
        #print('Reading with Scale: ', scale)
        if not offset:
            offset = [.0, .0, .0, .0]
        for i in range(uv_count):
            uv1 = struct.unpack('<h', self.__file__.read(2))[0] / 32768.0
            uv2 = struct.unpack('<h', self.__file__.read(2))[0] / 32768.0
            # if i in range(40):
            #    print(i, uv1, int((convuv1 / 2.0) * 65535.0))
            #    print(i, uv2, int((convuv2 / 2.0) * 65535.0))
            uvs.append(((offset[0] + scale[0] * uv1),
                        (offset[1] + scale[1] * uv2)))
            # uvs.append((convuv1, convuv2))
        return uvs

    def write_uvs_half(self, scale, offset, uvs):
        if not scale:
            scale = [1., 1., 1., 1.]
        if not offset:
            offset = [.0, .0, .0, .0]
        t = BytesIO()
        for i in range(len(uvs)):
            val1 = (uvs[i][0] - offset[0]) / scale[0]
            val2 = (uvs[i][1] - offset[1]) / scale[1]
            val1 = int(32768.0 * val1)
            val2 = int(32768.0 * val2)

            # if i < 40:
            #     print(val1, val2)
            uv1 = struct.pack('<2h', val1, val2)
            t.write(uv1)
        t.seek(0)
        self.__file__.write(t.read())

    def read_unknown(self, f):
        count = (self.size - 0x10) // 8
        for i in range(count):
            f.read(8)
        return None

    def tris_to_faces(self):
        chunkNum = len(self.data)
        faces = []
        for i in range(chunkNum):
            if len(self.data[i]) == 4:
                faces.append(self.data[i])
            elif len(self.data[i]) == 8:
                faces.append(self.data[i][0:3])
                faces.append(self.data[i][4:7])
        return faces

    def strips_to_faces(self):
        chunkNum = len(self.data)
        faces = []
        degenerate_faces = []
        for i in range(chunkNum):
            strip = self.data[i]
            faceNum = len(strip) - 2
            flag = True
            index = 0
            for j in range(faceNum):
                if flag:
                    tup = (strip[index], strip[index + 1], strip[index + 2])
                    if not (tup[0] == tup[1] or tup[1] == tup[2] or tup[0] == tup[2]):
                        faces.append(tup)
                    else:
                        degenerate_faces.append(tup)
                    index += 1
                    flag = not flag
                else:
                    tup = (strip[index + 1], strip[index], strip[index + 2])
                    if not (tup[0] == tup[1] or tup[1] == tup[2] or tup[0] == tup[2]):
                        faces.append(tup)
                    else:
                        degenerate_faces.append(tup)
                    index += 1
                    flag = not flag
        print('Degenerate Faces: ', len(degenerate_faces))
        return faces

    def read_blendindices_uint(self):
        count = (self.size - 0x10) // 4
        indices = []
        for i in range(count):
            entry = struct.unpack('4B', self.__file__.read(4))
            indices.append(entry)
        return indices

    def read_blendweights_unorm(self):
        count = (self.size - 0x10) // 4
        indices = []
        for i in range(count):
            entry = struct.unpack('4B', self.__file__.read(4))
            entry = [j / 255.0 for j in entry]
            indices.append(entry)
        return indices

    def read_colors_32uint(self):
        count = (self.size - 0x10) // 4
        print('\tReading Vertex Colors: ', count)
        indices = []
        for i in range(count):
            entry = struct.unpack('<I', self.__file__.read(4))
            indices.append(entry)
        return indices

    def read_colors_unorm(self):
        count = (self.size - 0x10) // 4
        print('\tReading Vertex Colors: ', count)
        indices = []
        for i in range(count):
            entry = struct.unpack('4B', self.__file__.read(4))
            entry = [float(j) for j in entry]
            indices.append(entry)
        return indices

    def wrap(self, val):
        if val < -32767:
            return min(32767, 65535 + val)
        elif val > 32767:
            return max(-65535 + val, -32767)
        else:
            return val

    def skip_section(self):
        self.__file__.seek(self.size - 0x10, 1)

    @staticmethod
    def calculate_normals(binormals, tangents):
        norms = []
        for i in range(len(binormals)):
            tangs = tangents[i]  # original tangent vector
            # length=sqrt(sum([v**2 for v in tangs])) #  tangent vector length
            # tangs=[v/length for v in tangs] # normalize vector

            binorms = binormals[i]  # original binormals vector
            # length=sqrt(sum([v**2 for v in binorms]))
            # binormal vector length
            # binorms=[v/length for v in binorms] # normalize vector

            # store cross product
            cross = (binorms[1] * tangs[2] - binorms[2] * tangs[1],
                     binorms[2] * tangs[0] - binorms[0] * tangs[2],
                     binorms[0] * tangs[1] - binorms[1] * tangs[0])
            norms.append((cross[0], cross[1], cross[2]))
            # print(cross)
        return norms


class Model2kPart:

    def __init__(self, name, start, count):
        self.name = name
        self.start = start
        self.count = count

# Create Mesh and Object Function


def createmesh(verts, faces, uvs, name, count, id, subname, colors, normal_flag, normals, loc):
    scn = bpy.context.scene
    mesh = bpy.data.meshes.new("mesh" + str(count))
    mesh.from_pydata(verts, [], faces)

    for i in range(len(uvs)):
        mesh.uv_textures.new(name='map' + str(i))
    for i in range(len(colors)):
        mesh.vertex_colors.new(name='col' + str(i))

    bm = bmesh.new()
    bm.from_mesh(mesh)

    # Create UV MAPS
    for i in range(len(uvs)):
        uvlayer = bm.loops.layers.uv['map' + str(i)]
        for f in bm.faces:
            for l in f.loops:
                l[uvlayer].uv.x = uvs[i][l.vert.index][0]
                l[uvlayer].uv.y = 1 - uvs[i][l.vert.index][1]

    # Create VERTEX COLOR MAPS

    for i in range(len(colors)):
        collayer = bm.loops.layers.color['col' + str(i)]
        for f in bm.faces:
            for l in f.loops:
                l[collayer].r, l[collayer].g, l[
                    collayer].b = colors[i][l.vert.index][0:3]

    # Pass Normals
    if normal_flag:
        print('Importing Normals')
        for i in range(len(normals)):
            bm.verts[i].normal = Vector(normals[i])

    # Finish up, write the bmesh back to the mesh
    bm.to_mesh(mesh)
    bm.free()  # free and prevent further access

    # if name.split(sep='_')[0]=='head':
    #   object=bpy.data.objects.new(name+'_'+str(id)+'_'+str(count)+'_'+subname,mesh)
    if name.split(sep='_')[0] in ['stadium', 'head']:
        object = bpy.data.objects.new(subname, mesh)
    else:
        # object = bpy.data.objects.new(
        #     name + '_' + str(id) + '_' + str(count), mesh)
        object = bpy.data.objects.new(name, mesh)

    object.location = loc

    scn.objects.link(object)

    return object.name


def model_import(scn, jsondata, modeldata, mindex, part_name):
    fType = struct.unpack('>I', modeldata.read(4))[0]
    print('File Type: ', hex(fType))

    # Parse Model Parts
    model_parts = []
    sect = Model2k(modeldata)
    index = mindex['Prim']
    mode = None
    start = 0
    base = 0
    for dataentry in index:
        print(' '.join(map(str, ('Model Part:', dataentry))))
        mesh = None

        try:
            mesh = index[dataentry]['Mesh']
        except:
            mesh = dataentry + str('_model')

        try:
            count = index[dataentry]['Count']
        except:
            count = model_parts[-1][0].count
            # count = None
            # pass

        try:
            base = index[dataentry]['Base']
            # start = base
        except:
            pass

        if not mode:
            try:
                mode = index[dataentry]['Type']
            except:
                mode = 'TRIANGLE_STRIP'

        part = Model2kPart(mesh, start, count)  # Create Part Object

        if mode == 'TRIANGLE_STRIP':
            print(' '.join(map(str, ('Target Strips: ', part.count))))
            print(
                ' '.join(map(str, ('Starting offset: ', hex(sect.tell())))))
            sect.data = sect.read_strips(part.start, part.count + 1, base)
            part_indices = sect.strips_to_faces()
        else:
            print(
                ' '.join(map(str, ('\tTarget Triangles: ', part.count))))
            print(
                ' '.join(map(str, ('\tStarting offset: ', hex(sect.tell())))))
            part_indices = sect.read_lists(part.start, part.count, base)

        print(' '.join(map(str, ('\t', mesh, count, start, mode, base))))
        start += count
        print('-' * 20)
        model_parts.append([part, part_indices])
        # break

    modeldata.seek(sect.size + 4)
    # Parsing the Vertex Data
    index = mindex['VertexFormat']
    try:
        scale = mindex['Radius'] / 100.0
    except:
        print("\tMissing Radius Parameter")
        scale = 1.0
    try:
        location = Vector(mindex['Center']).xzy
    except:
        print("\tMissing Center Parameter")
    location = Vector((0.0, 0.0, 0.0))

    # print(' '.join(map(str,('model: ', mindex))))

    verts, normals, binormals, tangents, uvs, blendindices, blendweights, colors = [
    ], [], [], [], [], [], [], []
    for dataentry in index:
        print(' '.join(map(str, ('dataentry', dataentry))))
        vformat = None
        offset = None
        scale = None
        if dataentry == 'POSITION0':
            # parsing vertices
            vformat = index[dataentry]['Format']
            try:
                locscale = index[dataentry]['Scale']
                # print(' '.join(map(str, ('Scale', locscale))))
            except:
                # print("Missing LocalScale Parameter")
                locscale = [1.0, 1.0, 1.0]
            sect = Model2k(modeldata)
            verts = sect.get_verts(vformat, locscale)
        elif dataentry == 'NORMAL0':
            # parsing vertices
            vformat = index[dataentry]['Format']
            sect = Model2k(modeldata)
            normals = sect.get_normals(vformat)
        elif dataentry == 'BINORMAL0':
            # parsing normals
            vformat = index[dataentry]['Format']
            sect = Model2k(modeldata)
            binormals = sect.get_verts(vformat, scale)
        elif dataentry == 'COLOR0':
            # parsing colors
            vformat = index[dataentry]['Format']
            print('\t', vformat)
            sect = Model2k(modeldata)
            colors.append(sect.get_colors(vformat))
        elif dataentry == 'TANGENT0':
            # parsing tangents
            vformat = index[dataentry]['Format']
            sect = Model2k(modeldata)
            tangents = sect.get_verts(vformat, scale)
        elif 'TEXCOORD' in dataentry:
            # parsing uvs
            vformat = index[dataentry]['Format']
            try:
                scale = index[dataentry]['Scale']
            except:
                print("\tMissing Scale Parameter")
            try:
                offset = index[dataentry]['Offset']
            except:
                print("\tMissing Offset Parameter")
            sect = Model2k(modeldata)
            uvstemp = sect.get_uvs(vformat, scale, offset)
            # Exclude weird formats:
            if uvstemp:
                if vformat in ['R32_UINT']:
                    # Have no idea what this section is, it is way lower than
                    # the vertices count
                    pass
                else:
                    uvs.append(uvstemp)
        elif dataentry == 'BLENDINDICES0':
            # parsing blendindices
            vformat = index[dataentry]['Format']
            sect = Model2k(modeldata)

            blendindices = sect.get_blendindices(vformat)
        elif dataentry == 'BLENDWEIGHT0':
            # parsing blendweights
            vformat = index[dataentry]['Format']
            sect = Model2k(modeldata)
            blendweights = sect.get_blendweights(vformat)
        # break  # RETURNING

    faces = []
    for part, indices in model_parts:
        faces.extend(indices)

    # print(' '.join(map(str,('Uvs length', len(uvs))))
    # ob_name = createmesh(verts, faces, uvs, part.name, 0, 0, '', [
    # ], [], [], Vector(mindex[model]['Center']).xzy)
    # Calculate normals
    if not normals:
        for i in range(len(verts)):
            normals.append((0.0, 1.0, 0.0))
    if (len(binormals) & len(tangents)):
        normals = Model2k.calculate_normals(binormals, tangents)
    if not uvs:
        temp = []
        for i in range(len(verts)):
            temp.append((0.0, 0.0))
        uvs.append(temp)

    print('Vertices Count: ',  len(verts))
    # normals = Model2k.calculate_normals(normals, tangents)
    print('Uvs length', len(uvs), 'Colors length', len(colors))
    # print(verts[0:10])
    ob_name = createmesh(verts, faces, uvs, part_name, 0, 0, '',
                         colors, [], [], location)

    ### Add Vertex Groups with the Model Parts ###
    ob = bpy.data.objects[ob_name]

    bpy.context.scene.objects.active = ob  # Set active object on scene

    ### Create Material Vertex Groups ###
    for part, indices in model_parts:
        vg = ob.vertex_groups.new(part.name)

        sel_verts = []
        for v0, v1, v2 in indices:
            sel_verts.extend([v0, v1, v2])
        vg.add(sel_verts, 1.0, 'ADD')

    if scn.nba2kxx_skeleton_flag:
        skeleton_import(jsondata, blendindices, blendweights, ob)


def skeleton_import(jsondata, blendindices, blendweights, ob):
    # Parsing the Bones
    index = jsondata['Model']['player']['Node']
    bones = []
    boneId = 0
    ### Store Bones ###
    for dataentry in index:
        Child = None
        Sibling = None
        Name = dataentry
        Position = None
        try:
            Child = index[dataentry]['Child']
        except:
            pass

        try:
            Sibling = index[dataentry]['Sibling']
        except:
            pass

        try:
            Position = 0.5 * Vector(index[dataentry]['Pos'])
            Position[0] *= -1.0
        except:
            pass

        bone = Bone(Sibling, Child, Name, boneId, Position)
        ob.vertex_groups.new(bone.Name)
        bones.append(bone)
        boneId += 1

    print('Total Bones Detected: ', len(bones))
    ### Create Groups for Bones ###
    v_blendgroups = []
    v_blendweights = []
    for i in range(len(bones)):
        v_blendgroups.append([])
    for i in range(len(bones)):
        v_blendweights.append(0)

    for v_id in range(len(blendindices)):
        vblendindices = blendindices[v_id]
        vblendweights = blendweights[v_id]

        for e in range(len(vblendindices)):
            v_bone_id = vblendindices[e]
            v_bone_weight = vblendweights[e]
            # v_blendgroups[bone_id].append(v_id)
            vg = ob.vertex_groups[bones[v_bone_id].Name]
            vg.add([v_id], v_bone_weight, 'ADD')

    bpy.ops.object.add(
        type='ARMATURE',
        enter_editmode=True,
        location=Vector(jsondata['Model']['player']['Center']).xzy)

    ob = bpy.data.objects['Armature']
    ob.show_x_ray = True
    ob.name = 'nba_bones'
    amt = ob.data
    amt.name = 'NBA_Amt'
    amt.show_axes = True

    # Create bones
    bpy.ops.object.mode_set(mode='EDIT')

    for b in bones:
        if b.Name not in amt.edit_bones:
            bone = amt.edit_bones.new(b.Name)
        else:
            bone = amt.edit_bones[b.Name]

        if b.Position:
            bone.tail = bone.head + b.Position.xzy
        else:
            bone.tail = bone.head + Vector([0.01, 0.01, 0.01])
        bone.use_connect = True

        if b.Child:
            cb = bones[b.Child]
            child = amt.edit_bones.new(cb.Name)
            child.parent = bone
            child.head = bone.tail

        if b.Sibling:
            sb = bones[b.Sibling]
            sibling = amt.edit_bones.new(sb.Name)
            sibling.parent = bone.parent
            sibling.head = bone.head

    bpy.ops.object.mode_set(mode='OBJECT')


def model_export(scn, ob, jsondata, modeldata):
    print('Exporting:', ob.name)
    modeldata.seek(0x4)  # Skip header
    sect = Model2k(modeldata)
    modeldata.seek(sect.size + 4)  # Skip Triangles
    print(modeldata.tell())
    mindex = jsondata['Model']
    for model in mindex:
        index = mindex[model]['VertexFormat']
        model = ModelEntry()
        model.parse_data(ob)

        for dataentry in index:
            print('dataentry', dataentry)
            vformat = None
            scale = None
            offset = None

            if dataentry == 'POSITION0':
                print("Writing Verts")
                # writing vertices
                vformat = index[dataentry]['Format']
                print(vformat)
                try:
                    scale = index[dataentry]['Scale']
                except:
                    print("\tMissing Scale Parameter")
                sect = Model2k(modeldata)
                # modeldata.seek(sect.size - 0x10, 1)
                print('Writing vertices: ', len(model.verts))
                sect.write_verts(vformat, scale, model.verts)
            elif dataentry == 'NORMAL0':
                print("Writing Normals")
                # writing normals
                vformat = index[dataentry]['Format']
                print('normals starting offset', modeldata.tell())
                sect = Model2k(modeldata)
                modeldata.seek(sect.size - 0x10, 1)
            elif dataentry == 'BINORMAL0':
                print(modeldata.tell())
                print("Writing Binormals")
                # writing normals
                vformat = index[dataentry]['Format']
                print('binormals starting offset', modeldata.tell())
                sect = Model2k(modeldata)
                modeldata.seek(sect.size - 0x10, 1)
            elif dataentry == 'COLOR0':
                print(modeldata.tell())
                print("Writing Color")
                # writing normals
                vformat = index[dataentry]['Format']
                print('color starting offset', modeldata.tell())
                sect = Model2k(modeldata)
                modeldata.seek(sect.size - 0x10, 1)
            elif dataentry == 'TANGENT0':
                print(modeldata.tell())
                print("Writing Tangents")
                # writing tangents
                vformat = index[dataentry]['Format']
                sect = Model2k(modeldata)
                modeldata.seek(sect.size - 0x10, 1)
            elif 'TEXCOORD' in dataentry:
                print("Writing Uvs", dataentry)
                # writing uvs
                vformat = index[dataentry]['Format']
                try:
                    scale = index[dataentry]['Scale']
                except:
                    print("Missing Scale Parameter")
                try:
                    offset = index[dataentry]['Offset']
                except:
                    print("Missing Offset Parameter")
                sect = Model2k(modeldata)
                id = int(dataentry.split('TEXCOORD')[-1])
                print('Model UV Layer #', id)
                if id < len(model.uvs):
                    sect.write_uvs(vformat, scale, offset, model.uvs[id])
                else:
                    sect.skip_section()
            elif dataentry == 'BLENDINDICES0':
                # parsing blendindices
                vformat = index[dataentry]['Format']
                sect = Model2k(modeldata)
                modeldata.seek(sect.size - 0x10, 1)
            elif dataentry == 'BLENDWEIGHT0':
                # parsing blendweights
                vformat = index[dataentry]['Format']
                sect = Model2k(modeldata)
                modeldata.seek(sect.size - 0x10, 1)

    modeldata.close()


# -----------------Blender OPERATORS--------------
class nba_model_import(bpy.types.Operator):
    bl_idname = "mesh.nba2kxx_model_import"
    bl_label = "Import"
    bl_description = 'Click to Import Model'

    def invoke(self, context, event):
        scn = bpy.context.scene
        # Get json data for model file detection
        f = open(scn.nba_json_filepath, 'r')
        dirname = os.path.dirname(scn.nba_json_filepath)
        txtdata = f.read()
        f.close()
        jsondata = json_parser.NbaJsonParser(txtdata)
        for mindex in jsondata['Model']:
            binaryname = jsondata['Model'][mindex]['Binary']
            print('*' * 10)
            print(mindex, binaryname)
            try:
                modeldata = BytesIO()  # Prepare binary file
                f = open(os.path.join(dirname,
                                      binaryname), 'rb')
                modeldata.write(f.read())
                f.close()
                modeldata.seek(0)
                model_import(scn, jsondata, modeldata,
                             jsondata['Model'][mindex], mindex)

            except IOError:
                print('Missing Binary Name')

        return{'FINISHED'}


class nba_model_export(bpy.types.Operator):
    bl_idname = "mesh.nba2kxx_model_export"
    bl_label = "Export"
    bl_description = 'Click to Export Model'

    def invoke(self, context, event):
        scn = context.scene

        try:
            f = open(scn.nba_json_filepath, 'r')
            txtdata = f.read()
            f.close()
            jsondata = json_parser.NbaJsonParser(txtdata)
            dirname = os.path.dirname(scn.nba_json_filepath)
        except:
            self.report({'ERROR'}, 'Error in .json file')
            return {'CANCELLED'}

        for mindex in jsondata['Model']:
            try:
                binaryname = jsondata['Model'][mindex]['Binary']
                newfilepath = os.path.join(scn.nba_export_filepath, binaryname)
                copyfile(os.path.join(dirname, binaryname), newfilepath)
            except FileNotFoundError:
                self.report({'ERROR'}, 'File Not Found')
                return {'CANCELLED'}

            try:
                modeldata = open(newfilepath, 'r+b')
            except FileNotFoundError:
                self.report({'ERROR'}, 'File Not Found')
                return {'CANCELLED'}

            try:
                ob = scn.objects[mindex]
            except:
                self.report({'ERROR'}, 'Missing Object', mindex)
                continue

            model_export(scn, ob, jsondata, modeldata)
            # ob.location = back_loc
            # print('Restoring Location: ', ob.location, back_loc)
        return{'FINISHED'}

# -----------------MISC OPERATORS-------------- #


class visit_url(bpy.types.Operator):
    bl_idname = "system.visit_url"
    bl_label = ""

    def invoke(self, context, event):
        webbrowser.open(
            url='http://forums.nba-live.com/viewtopic.php?f=221&t=100276')
        return{'FINISHED'}


class visit_url_blog(bpy.types.Operator):
    bl_idname = "system.visit_url_blog"
    bl_label = ""

    def invoke(self, context, event):
        webbrowser.open(url='http://3dgamedevblog.com')
        return{'FINISHED'}


class visit_url_donate(bpy.types.Operator):
    bl_idname = "system.visit_url_donate"
    bl_label = ""

    def invoke(self, context, event):
        webbrowser.open(
            url='https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=arianos10@gmail.com&lc=US&item_name=3dgamedevblog&amount=5.00&currency_code=USD&no_note=0&bn=PP-DonationsBF:btn_donateCC_LG.gif:NonHostedGuest')
        return{'FINISHED'}


# -----------------Blender PROPERTIES--------------

bpy.types.Scene.nba_json_filepath = bpy.props.StringProperty(
    name="metadata file (scne/json)",
    description='Select .json file',
    subtype='FILE_PATH',
)

bpy.types.Scene.nba_export_filepath = bpy.props.StringProperty(
    name="Export Path",
    description='Set the export path for your model',
    subtype='FILE_PATH',
)

bpy.types.Scene.nba2kxx_skeleton_flag = bpy.props.BoolProperty(
    name="Import Model Skeleton",
    description='Select wether to Import the model skeleton or not',
    default=False
)


def register():
    bpy.utils.register_module(__name__)
    pass


def unregister():
    bpy.utils.unregister_module(__name__)
    pass

if __name__ == "__main__":
    register()

